﻿using AdvanceSoftware.ExcelCreator;
using System.Data;

namespace EC11WebSample.Models
{
    public class ReportService
    {
        private readonly IWebHostEnvironment environment;
        private readonly string templatePath;

        public ReportService(IWebHostEnvironment environment)
        {
            this.environment = environment;
            templatePath = Path.Combine(environment.WebRootPath, "resource", "template.xlsx");
        }

        private DataTable GetData()
        {
            var table = new DataTable();

            table.Columns.Add("**Company", typeof(string));
            table.Columns.Add("**Delivery", typeof(string));
            table.Columns.Add("**Item", typeof(string));
            table.Columns.Add("**Quantity", typeof(int));
            table.Columns.Add("**UnitPrice", typeof(int));
            table.Columns.Add("**Price", typeof(int));

            table.Rows.Add("○○株式会社", "西部配送センター", "ハウスキット１", 5, 3080, 15400);
            table.Rows.Add("○○株式会社", "西部配送センター", "ハウスキット２", 5, 1760, 8800);
            table.Rows.Add("○○株式会社", "西部配送センター", "プッシュ式型抜きセット", 10, 110, 1100);
            table.Rows.Add("○○株式会社", "西部配送センター", "クレイガン", 5, 1980, 9900);
            table.Rows.Add("○○株式会社", "西部配送センター", "樹脂粘土", 100, 660, 66000);
            table.Rows.Add("○○株式会社", "西部配送センター", "樹脂粘土", 100, 660, 66000);
            table.Rows.Add("○○株式会社", "西部配送センター", "樹脂粘土", 100, 660, 66000);
            table.Rows.Add("○○株式会社", "西部配送センター", "樹脂粘土", 100, 660, 66000);
            table.Rows.Add("○○株式会社", "東部配送センター", "カラーフェルト30cm 5色", 200, 341, 68200);
            table.Rows.Add("○○株式会社", "東部配送センター", "ぬい針6本セット", 200, 385, 77000);
            table.Rows.Add("○○株式会社", "東部配送センター", "カラーミシン糸 12色", 100, 484, 48400);
            table.Rows.Add("○○株式会社", "東部配送センター", "プラスティックスナップ120組", 15, 3080, 46200);
            table.Rows.Add("○○株式会社", "東部配送センター", "プラスティックスナップ120組", 15, 3080, 46200);
            table.Rows.Add("○○株式会社", "東部配送センター", "プラスティックスナップ120組", 15, 3080, 46200);
            table.Rows.Add("○○株式会社", "東部配送センター", "プラスティックスナップ120組", 15, 3080, 46200);
            table.Rows.Add("○○株式会社", "東部配送センター", "プラスティックスナップ120組", 15, 3080, 46200);
            table.Rows.Add("○○株式会社", "南部配送センター", "プラスティックスナップ120組", 15, 3080, 46200);
            table.Rows.Add("○○株式会社", "南部配送センター", "プラスティックスナップ120組", 15, 3080, 46200);
            table.Rows.Add("○○株式会社", "南部配送センター", "プラスティックスナップ120組", 15, 3080, 46200);
            table.Rows.Add("○○株式会社", "南部配送センター", "プラスティックスナップ120組", 15, 3080, 46200);
            table.Rows.Add("○○株式会社", "南部配送センター", "プラスティックスナップ120組", 15, 3080, 46200);
            table.Rows.Add("○○株式会社", "南部配送センター", "プラスティックスナップ120組", 15, 3080, 46200);
            table.Rows.Add("○○株式会社", "南部配送センター", "プラスティックスナップ120組", 15, 3080, 46200);
            table.Rows.Add("○○株式会社", "北部配送センター", "プラスティックスナップ120組", 15, 3080, 46200);
            
            return table;
        }

        public byte[] DataList()
        {
            using (var ms = new MemoryStream())
            using (var creator = new Creator())
            {
                creator.CreateBook(ms, 1, xlsxVersion.ver2021);

                var table = GetData();
                var rowIndex = 0;
                for (var colIndex = 0; colIndex < table.Columns.Count; colIndex++)
                {
                    creator.Pos(colIndex, rowIndex).Value = table.Columns[colIndex].ColumnName;
                }

                rowIndex++;
                foreach (DataRow row in table.Rows)
                {
                    for (var colIndex = 0; colIndex < table.Columns.Count; colIndex++)
                    {
                        creator.Pos(colIndex, rowIndex).Value = row[colIndex];
                    }
                    rowIndex++;
                }

                creator.CloseBook(true);

                return ms.ToArray();
            }
        }

        
        public byte[] Invoice()
        {
            using (var creator = new Creator())
            {
                var outputPath = Path.Combine(
                    environment.WebRootPath,
                    "invoice",
                    $"invoice{DateTime.Now.ToString("yyyyMMddHHmmss")}.xlsx"
                    );
                creator.OpenBook(outputPath, templatePath);

                var table = GetData();
                var deliveryGroups = table.AsEnumerable().GroupBy(r => r.Field<string>("**Delivery"));
                foreach (var delivery in deliveryGroups)
                {
                    InvoiceDetail(creator, delivery.ToArray());
                }

                creator.DeleteSheet(creator.SheetNo2("レイアウト"), 1);
                creator.CloseBook(true);

                return File.ReadAllBytes(outputPath);
            }
        }

        private void InvoiceDetail(Creator creator, DataRow[] delivery) 
        {
            const int TEMPLATE_ROWINDEX = 10;
            int detailCount = 0;
            int detailCountTotal = 0;
            int sheetCountTotal = 0;

            foreach (DataRow row in delivery)
            {
                if (detailCount == 0)
                {
                    sheetCountTotal++;
                    var sheetCount = creator.SheetCount;
                    var sheetName = row["**DeliVery"] + sheetCountTotal.ToString();
                    creator.CopySheet(creator.SheetNo2("レイアウト"), sheetCount, sheetName);
                    creator.SheetNo = sheetCount;
                    creator.Cell("**Company").Value = row["**Company"];
                    creator.Cell("**Delivery").Value = row["**Delivery"];
                    creator.RowCopy(TEMPLATE_ROWINDEX);
                }

                detailCount++;
                detailCountTotal++;
                creator.RowInsertCopied(TEMPLATE_ROWINDEX + detailCount, 1);
                creator.Cell("**Item", 0, detailCount).Value = row["**Item"];
                creator.Cell("**Quantity", 0, detailCount).Value = row["**Quantity"];
                creator.Cell("**UnitPrice", 0, detailCount).Value = row["**UnitPrice"];
                creator.Cell("**Price", 0, detailCount).Value = row["**Price"];

                if (detailCount == 5 || delivery.Count() == detailCountTotal)
                {
                    creator.RowDelete(TEMPLATE_ROWINDEX, 1);
                    int sum = TEMPLATE_ROWINDEX + detailCount;
                    creator.Cell("**TotalPrice").Func("=SUM(J11:J" + sum + ")", null);
                    detailCount = 0;
                }
            }
        }
    }
}
